Imports CQG

Public Class AutoTrader

   Private m_CEL As CQGCEL
   Private m_lvAutoTrades As ListView
   Private m_lvOrders As ListView
   Private m_Symbol As String
   Private m_Instrument As CQGInstrument
   Private m_OpenPositions As Integer
   Private m_Name As String
   Private m_Account As CQGAccount
   Private m_TradingSystem As CQGTradingSystem
   Private m_IsNameSelected As Boolean
   Private m_IsAccountSelected As Boolean
   Private m_IsTSResolved As Boolean
   Private m_IsSymbolResolved As Boolean
   Private m_IsActivated As Boolean
   Private m_Row As Integer
   Private m_tradesRowData As TradesRowData

   Private Enum eAutoTradeColumns
      atcTurnedOnAt = 1
      atcOpenPositions
      atcName
      atcTradingSystem
      atcSymbol
      atcAccountID
      atcAccountName
      atcFullName
      atcPreferences
      atcParameters
   End Enum

   Public Sub New(ByVal CEL As CQGCEL, _
               ByVal lvAT As ListView, _
               ByVal lvO As ListView)

      m_CEL = CEL
      m_lvAutoTrades = lvAT
      m_lvOrders = lvO

      m_IsNameSelected = False
      m_IsAccountSelected = False
      m_IsTSResolved = False
      m_IsSymbolResolved = False
      m_IsActivated = False

      m_OpenPositions = Integer.MinValue
      m_tradesRowData = New TradesRowData

      Dim item As ListViewItem = m_lvAutoTrades.Items.Add(New ListViewItem)
      For i As Integer = eAutoTradeColumns.atcTurnedOnAt To eAutoTradeColumns.atcParameters
         item.SubItems.Add("")
      Next

   End Sub

   Public Function GetPreferences() As String
      Dim sPref As String = ""

      If Not m_TradingSystem Is Nothing Then
         With m_TradingSystem.Request
            If .HistoricalPeriod <> eHistoricalPeriod.hpUndefined Then
               sPref = HistoricalPeriodToString(.HistoricalPeriod) & ", "
            Else
               sPref = .IntradayPeriod & " Min, "
            End If

            sPref &= ("CNT:" & ContinuationTypeToString(.Continuation) & ", ")
            sPref &= ("DBE:" & .DaysBeforeExpiration & ", ")
            sPref &= ("ECL:" & .EqualizeCloses & ", ")
            sPref &= ("SFL:" & .SessionsFilter.ToString())
         End With
      End If

      Return sPref
   End Function

   Public Function GetParameters() As String
      Dim sParams As String = ""
      Dim sParamName As String

      If Not m_TradingSystem Is Nothing Then
         With m_TradingSystem.Request
            For i As Integer = 0 To .Definition.ParameterDefinitions.Count - 1
               sParamName = .Definition.ParameterDefinitions.Item(i).Name
               sParams &= (sParamName & ":" & .Parameter(sParamName).ToString & ", ")
            Next
         End With
      End If

      Return sParams
   End Function

   Public WriteOnly Property Name() As String
      Set(ByVal newVal As String)
         m_Name = newVal
         m_IsNameSelected = (newVal.Trim.Length > 0)
         m_lvAutoTrades.Items(m_Row).SubItems(eAutoTradeColumns.atcName).Text = newVal
      End Set
   End Property

   Public WriteOnly Property TradingSystemDefName() As String
      Set(ByVal newVal As String)
         m_lvAutoTrades.Items(m_Row).SubItems(eAutoTradeColumns.atcTradingSystem).Text = newVal
      End Set
   End Property

   Public WriteOnly Property Account() As CQGAccount
      Set(ByVal newVal As CQGAccount)
         m_Account = newVal
         m_IsAccountSelected = (Not newVal Is Nothing)
         If m_IsAccountSelected Then
            m_lvAutoTrades.Items(m_Row).SubItems(eAutoTradeColumns.atcAccountID).Text = newVal.GWAccountID.ToString()
            m_lvAutoTrades.Items(m_Row).SubItems(eAutoTradeColumns.atcAccountName).Text = newVal.GWAccountName
         Else
            m_lvAutoTrades.Items(m_Row).SubItems(eAutoTradeColumns.atcAccountID).Text = ""
            m_lvAutoTrades.Items(m_Row).SubItems(eAutoTradeColumns.atcAccountName).Text = ""
         End If
      End Set
   End Property

   Public WriteOnly Property TradingSystem() As CQGTradingSystem
      Set(ByVal newVal As CQGTradingSystem)
         m_TradingSystem = newVal
         m_IsTSResolved = (Not newVal Is Nothing)
         If m_IsTSResolved Then
            m_tradesRowData.SetData(m_TradingSystem.TradesRows(m_TradingSystem.TradesRows.Count - 1))
            m_lvAutoTrades.Items(m_Row).SubItems(eAutoTradeColumns.atcPreferences).Text = GetPreferences()
            m_lvAutoTrades.Items(m_Row).SubItems(eAutoTradeColumns.atcParameters).Text = GetParameters()
         Else
            m_lvAutoTrades.Items(m_Row).SubItems(eAutoTradeColumns.atcPreferences).Text = ""
            m_lvAutoTrades.Items(m_Row).SubItems(eAutoTradeColumns.atcParameters).Text = ""
         End If
      End Set
   End Property

   Public Property Symbol() As String
      Get
         Return m_Symbol
      End Get
      Set(ByVal newVal As String)
         m_Symbol = newVal
         m_lvAutoTrades.Items(m_Row).SubItems(eAutoTradeColumns.atcSymbol).Text = newVal
         m_lvAutoTrades.Items(m_Row).SubItems(eAutoTradeColumns.atcFullName).Text = ""

         If newVal.Trim.Length > 0 Then
            m_CEL.NewInstrument(newVal)
         End If
      End Set
   End Property

   Public WriteOnly Property Instrument() As CQGInstrument
      Set(ByVal newVal As CQGInstrument)
         m_Instrument = newVal
         m_IsSymbolResolved = (Not newVal Is Nothing)
         If m_IsSymbolResolved Then
            m_lvAutoTrades.Items(m_Row).SubItems(eAutoTradeColumns.atcFullName).Text = m_Instrument.FullName
         Else
            m_lvAutoTrades.Items(m_Row).SubItems(eAutoTradeColumns.atcFullName).Text = N_A
         End If
      End Set
   End Property

   Public WriteOnly Property Row() As Integer
      Set(ByVal newVal As Integer)
         m_Row = newVal
      End Set
   End Property

   Private Function GetReadyStatus() As Boolean

      Return (m_IsNameSelected And m_IsAccountSelected And m_IsTSResolved And m_IsSymbolResolved And m_OpenPositions <> Integer.MinValue)

   End Function

   Public ReadOnly Property IsSymbolResolved() As Boolean
      Get
         Return m_IsSymbolResolved
      End Get
   End Property

   Public Property TradeRowData() As TradesRowData
      Get
         Return m_tradesRowData
      End Get
      Set(ByVal newVal As TradesRowData)
         m_tradesRowData = newVal
      End Set
   End Property

   Public Sub Activate(Optional ByRef newVal As CheckState = CheckState.Checked)
      If newVal = CheckState.Checked Then
         If GetReadyStatus() Then
            m_IsActivated = True
            m_lvAutoTrades.Items(m_Row).SubItems(eAutoTradeColumns.atcTurnedOnAt).Text = Now.ToString
         Else
            newVal = CheckState.Unchecked
         End If
      Else
         m_IsActivated = False
         m_lvAutoTrades.Items(m_Row).SubItems(eAutoTradeColumns.atcTurnedOnAt).Text = ""
      End If

   End Sub

   Public Sub PositionsChanged(ByVal tradingSystem As CQGTradingSystem)
      If Not tradingSystem Is Nothing Then
         Dim newVal As Integer = CInt(tradingSystem.Statistics(tradingSystem.Statistics.Count - 1).OpenPosition)
         If m_CEL.IsValid(newVal) Then
            m_lvAutoTrades.Items(m_Row).SubItems(eAutoTradeColumns.atcOpenPositions).Text = newVal.ToString
         Else
            If m_lvAutoTrades.Items(m_Row).SubItems(eAutoTradeColumns.atcOpenPositions).Text.Trim.Length = 0 Then
               m_lvAutoTrades.Items(m_Row).SubItems(eAutoTradeColumns.atcOpenPositions).Text = N_A
            End If
            Exit Sub
         End If

         Dim delta As Integer = 0
         If m_OpenPositions <> Integer.MinValue Then
            delta = newVal - m_OpenPositions
         End If

         m_OpenPositions = newVal

         If m_IsActivated Then
            PlaceOrder(delta)
         End If
      End If

   End Sub

   Private Sub PlaceOrder(ByVal quantity_ As Integer)

      If quantity_ = 0 Then
         Exit Sub
      End If

      Dim order_ As CQGOrder = m_CEL.CreateOrder(eOrderType.otMarket, m_Instrument, m_Account, _
                                                      quantity_, , , , "AUTOTRADE")
      order_.Place()

      ExecutedOrders(order_)

   End Sub

   Private Sub ExecutedOrders(ByVal order As CQGOrder)

      Dim item As ListViewItem = m_lvOrders.Items.Add(New ListViewItem)
      For i As Integer = eExecutedOrdersColumns.eocAutoTradeName To eExecutedOrdersColumns.eocGUID
         item.SubItems.Add("")
      Next

      Dim iRow As Integer = m_lvOrders.Items.Count - 1
      m_lvOrders.Items(iRow).SubItems(eExecutedOrdersColumns.eocAutoTradeName).Text = m_Name
      m_lvOrders.Items(iRow).SubItems(eExecutedOrdersColumns.eocAccount).Text = order.Account.GWAccountName
      m_lvOrders.Items(iRow).SubItems(eExecutedOrdersColumns.eocInstrument).Text = order.InstrumentName
      m_lvOrders.Items(iRow).SubItems(eExecutedOrdersColumns.eocSide).Text = _
                                    IIf(order.Quantity < 0, "Sell", "Buy").ToString()
      m_lvOrders.Items(iRow).SubItems(eExecutedOrdersColumns.eocPlaceTime).Text = N_A
      m_lvOrders.Items(iRow).SubItems(eExecutedOrdersColumns.eocQuantity).Text = _
                                    GetValueAsString(Math.Abs(order.Quantity), m_CEL)
      m_lvOrders.Items(iRow).SubItems(eExecutedOrdersColumns.eocFilledQuantity).Text = _
                                    GetValueAsString(Math.Abs(order.FilledQuantity), m_CEL)
      m_lvOrders.Items(iRow).SubItems(eExecutedOrdersColumns.eocOrderType).Text = order.Type.ToString()
      m_lvOrders.Items(iRow).SubItems(eExecutedOrdersColumns.eocOrderID).Text = GetValueAsString(order.OriginalOrderID, m_CEL)
      m_lvOrders.Items(iRow).SubItems(eExecutedOrdersColumns.eocGWStatus).Text = GetValueAsString(order.GWStatus, m_CEL)
      m_lvOrders.Items(iRow).SubItems(eExecutedOrdersColumns.eocState).Text = GetValueAsString(order.State, m_CEL)
      m_lvOrders.Items(iRow).SubItems(eExecutedOrdersColumns.eocUEName).Text = order.UEName
      m_lvOrders.Items(iRow).SubItems(eExecutedOrdersColumns.eocGUID).Text = order.GUID

   End Sub

   Private Function HistoricalPeriodToString(ByVal historicalPeriod As eHistoricalPeriod) As String
      Dim sHistoricalPeriod As String = ""

      Select Case historicalPeriod
         Case eHistoricalPeriod.hpUndefined
            sHistoricalPeriod = "Undefined"
         Case eHistoricalPeriod.hpDaily
            sHistoricalPeriod = "Daily"
         Case eHistoricalPeriod.hpWeekly
            sHistoricalPeriod = "Weekly"
         Case eHistoricalPeriod.hpMonthly
            sHistoricalPeriod = "Monthly"
         Case eHistoricalPeriod.hpQuarterly
            sHistoricalPeriod = "Quarterly"
         Case eHistoricalPeriod.hpSemiannual
            sHistoricalPeriod = "Semiannual"
         Case eHistoricalPeriod.hpYearly
            sHistoricalPeriod = "Yearly"
      End Select

      Return sHistoricalPeriod
   End Function

   Private Function ContinuationTypeToString(ByVal continuationType As eTimeSeriesContinuationType) As String
      Dim sContinuationType As String = ""

      Select Case continuationType
         Case eTimeSeriesContinuationType.tsctNoContinuation
            sContinuationType = "NC"
         Case eTimeSeriesContinuationType.tsctStandard
            sContinuationType = "St"
         Case eTimeSeriesContinuationType.tsctStandardByMonth
            sContinuationType = "StBM"
         Case eTimeSeriesContinuationType.tsctActive
            sContinuationType = "Ac"
         Case eTimeSeriesContinuationType.tsctActiveByMonth
            sContinuationType = "AcBM"
         Case eTimeSeriesContinuationType.tsctAdjusted
            sContinuationType = "Ad"
         Case eTimeSeriesContinuationType.tsctAdjustedByMonth
            sContinuationType = "AdBM"
      End Select

      Return sContinuationType
   End Function

End Class
