Imports CQG

Public Class TradesRowData

   ''' <summary>
   ''' Trade row timestamp.
   ''' </summary>   
   Private m_TradeRowTimestamp As DateTime

   ''' <summary>
   ''' Trade row timestamp offset.
   ''' </summary>
   Private m_TradeRowTimestampOffset As Long

   ''' <summary>
   ''' Flag indicating whether trade row is in signaled state.
   ''' </summary>
   Private m_IsSignaled As Boolean

   ''' <summary>
   ''' Initializes trades row data.
   ''' </summary>
   Public Sub New()

      m_TradeRowTimestampOffset = Long.MinValue
      m_IsSignaled = False

   End Sub

   ''' <summary>
   ''' Gets trades row timestamp.
   ''' </summary>
   Public ReadOnly Property Timestamp() As DateTime

      Get
         Timestamp = m_TradeRowTimestamp
      End Get

   End Property

   ''' <summary>
   ''' Gets trades row timestamp offset.
   ''' </summary>
   Public ReadOnly Property TimestampOffset() As Long

      Get
         TimestampOffset = m_TradeRowTimestampOffset
      End Get

   End Property

   ''' <summary>
   ''' Gets flag indicating whether trades row is in signaled state ot not.
   ''' </summary>
   Public ReadOnly Property Signaled() As Boolean

      Get
         Signaled = m_IsSignaled
      End Get

   End Property

   ''' <summary>
   ''' Sets trades row data.
   ''' </summary>
   Public Sub SetData(ByVal tradesRow As CQGTradingSystemTradesRow)

      If tradesRow Is Nothing Then
         Exit Sub
      End If

      m_TradeRowTimestamp = tradesRow.Timestamp
      m_TradeRowTimestampOffset = tradesRow.TimestampOffset
      m_IsSignaled = False

      Dim trade As CQGTradingSystemTrade
      For Each trade In tradesRow
         If trade.TradeEntry.Signal Then
            m_IsSignaled = True
            Exit Sub
         End If

         Dim tradeExit As CQGTradeExit
         For Each tradeExit In trade.TradeExits
            If tradeExit.Signal Then
               m_IsSignaled = True
               Exit Sub
            End If
         Next tradeExit

      Next trade

   End Sub

End Class
